@@ -284,8 +284,8 @@ static void mlt_consumer_property_changed( mlt_properties owner, mlt_consumer se
  * \private \memberof mlt_consumer_s
  * \param listener a function pointer that will be invoked
  * \param owner the events object that will be passed to \p listener
- * \param self  a service that will be passed to \p listener
- * \param args an array of pointers - the first entry is passed as a string to \p listener
+ * \param self a service that will be passed to \p listener
+ * \param args an array of pointers - the first entry is passed as a frame to \p listener
  */
 
 static void mlt_consumer_frame_show( mlt_listener listener, mlt_properties owner, mlt_service self, void **args )
@@ -301,8 +301,8 @@ static void mlt_consumer_frame_show( mlt_listener listener, mlt_properties owner
  * \private \memberof mlt_consumer_s
  * \param listener a function pointer that will be invoked
  * \param owner the events object that will be passed to \p listener
- * \param self  a service that will be passed to \p listener
- * \param args an array of pointers - the first entry is passed as a string to \p listener
+ * \param self a service that will be passed to \p listener
+ * \param args an array of pointers - the first entry is passed as a frame to \p listener
  */
 
 static void mlt_consumer_frame_render( mlt_listener listener, mlt_properties owner, mlt_service self, void **args )
@@ -449,6 +449,9 @@ int mlt_consumer_start( mlt_consumer self )
 		mlt_properties_set_data( properties, "test_card_producer", NULL, 0, NULL, NULL );
 	}
 
+	// The profile could have changed between a stop and a restart.
+	apply_profile_properties( self, mlt_service_profile( MLT_CONSUMER_SERVICE(self) ), properties );
+
 	// Set the frame duration in microseconds for the frame-dropping heuristic
 	int frame_rate_num = mlt_properties_get_int( properties, "frame_rate_num" );
 	int frame_rate_den = mlt_properties_get_int( properties, "frame_rate_den" );
@@ -666,6 +669,8 @@ static void *consumer_read_ahead_thread( void *arg )
 			afmt = mlt_audio_float;
 		else if ( !strcmp( format, "f32le" ) )
 			afmt = mlt_audio_f32le;
+		else if ( !strcmp( format, "u8" ) )
+			afmt = mlt_audio_u8;
 	}
 	int counter = 0;
 	double fps = mlt_properties_get_double( properties, "fps" );
