@@ -131,10 +131,15 @@ int mlt_frame_set_aspect_ratio( mlt_frame self, double value )
 }
 
 /** Get the time position of this frame.
+ *
+ * This position is not necessarily the position as the original
+ * producer knows it. It could be the position that the playlist,
+ * multitrack, or tractor producer set.
  *
  * \public \memberof mlt_frame_s
  * \param self a frame
  * \return the position
+ * \see mlt_frame_original_position
  */
 
 mlt_position mlt_frame_get_position( mlt_frame self )
@@ -143,6 +148,21 @@ mlt_position mlt_frame_get_position( mlt_frame self )
 	return pos < 0 ? 0 : pos;
 }
 
+/** Get the original time position of this frame.
+ *
+ * This is the position that the original producer set on the frame.
+ *
+ * \public \memberof mlt_frame_s
+ * \param self a frame
+ * \return the position
+ */
+
+mlt_position mlt_frame_original_position( mlt_frame self )
+{
+	int pos = mlt_properties_get_position( MLT_FRAME_PROPERTIES( self ), "original_position" );
+	return pos < 0 ? 0 : pos;
+}
+
 /** Set the time position of this frame.
  *
  * \public \memberof mlt_frame_s
@@ -153,6 +173,9 @@ mlt_position mlt_frame_get_position( mlt_frame self )
 
 int mlt_frame_set_position( mlt_frame self, mlt_position value )
 {
+	// Only set the original_position the first time.
+	if ( ! mlt_properties_get( MLT_FRAME_PROPERTIES( self ), "original_position" ) )
+		mlt_properties_set_position( MLT_FRAME_PROPERTIES( self ), "original_position", value );
 	return mlt_properties_set_position( MLT_FRAME_PROPERTIES( self ), "_position", value );
 }
 
@@ -606,6 +629,7 @@ const char * mlt_audio_format_name( mlt_audio_format format )
 		case mlt_audio_s32le:  return "s32le";
 		case mlt_audio_float:  return "float";
 		case mlt_audio_f32le:  return "f32le";
+		case mlt_audio_u8:     return "u8";
 	}
 	return "invalid";
 }
@@ -629,6 +653,7 @@ int mlt_audio_format_size( mlt_audio_format format, int samples, int channels )
 		case mlt_audio_s32:    return samples * channels * sizeof( int32_t );
 		case mlt_audio_f32le:
 		case mlt_audio_float:  return samples * channels * sizeof( float );
+		case mlt_audio_u8:     return samples * channels;
 	}
 	return 0;
 }
@@ -798,6 +823,8 @@ unsigned char *mlt_frame_get_waveform( mlt_frame self, int w, int h )
 
 	// Make an 8-bit buffer large enough to hold rendering
 	int size = w * h;
+	if ( size <= 0 )
+		return NULL;
 	unsigned char *bitmap = ( unsigned char* )mlt_pool_alloc( size );
 	if ( bitmap != NULL )
 		memset( bitmap, 0, size );
