@@ -231,6 +231,8 @@ mlt_profile mlt_profile_load_properties( mlt_properties properties )
 mlt_profile mlt_profile_load_string( const char *string )
 {
 	mlt_properties properties = mlt_properties_new();
+	mlt_profile profile = NULL;
+
 	if ( properties )
 	{
 		const char *p = string;
@@ -241,8 +243,10 @@ mlt_profile mlt_profile_load_string( const char *string )
 			p = strchr( p, '\n' );
 			if ( p ) p++;
 		}
+		profile = mlt_profile_load_properties( properties );
+		mlt_properties_close( properties );
 	}
-	return mlt_profile_load_properties( properties );
+	return profile;
 }
 
 /** Get the video frame rate as a floating point value.
