@@ -47,7 +47,10 @@ struct mlt_properties_s
 
 extern int mlt_properties_init( mlt_properties, void *child );
 extern mlt_properties mlt_properties_new( );
+extern int mlt_properties_set_lcnumeric( mlt_properties, const char *locale );
+extern const char* mlt_properties_get_lcnumeric( mlt_properties self );
 extern mlt_properties mlt_properties_load( const char *file );
+extern int mlt_properties_preset( mlt_properties self, const char *name );
 extern int mlt_properties_inc_ref( mlt_properties self );
 extern int mlt_properties_dec_ref( mlt_properties self );
 extern int mlt_properties_ref_count( mlt_properties self );
@@ -85,5 +88,6 @@ extern mlt_properties mlt_properties_parse_yaml( const char *file );
 extern char *mlt_properties_serialise_yaml( mlt_properties self );
 extern void mlt_properties_lock( mlt_properties self );
 extern void mlt_properties_unlock( mlt_properties self );
+extern char *mlt_properties_get_time( mlt_properties, const char* name, mlt_time_format );
 
 #endif
