@@ -124,3 +124,7 @@ Filter *Service::filter( int index )
 	return result == NULL ? NULL : new Filter( result );
 }
 
+void Service::set_profile( Profile &profile )
+{
+	mlt_service_set_profile( get_service( ), profile.get_profile( ) );
+}
