@@ -1,6 +1,6 @@
 /*
  * factory.c -- the factory method interfaces
- * Copyright (C) 2003-2004 Ushodaya Enterprises Limited
+ * Copyright (C) 2003-2012 Ushodaya Enterprises Limited
  * Author: Charles Yates <charles.yates@pandora.be>
  *
  * This library is free software; you can redistribute it and/or
@@ -43,51 +43,44 @@ extern mlt_producer producer_avformat_init( mlt_profile profile, const char *ser
 #include <libavcodec/opt.h>
 #endif
 
+#if LIBAVUTIL_VERSION_INT < ((51<<16)+(12<<8)+0)
+#define AV_OPT_TYPE_FLAGS    FF_OPT_TYPE_FLAGS
+#define AV_OPT_TYPE_INT      FF_OPT_TYPE_INT
+#define AV_OPT_TYPE_INT64    FF_OPT_TYPE_INT64
+#define AV_OPT_TYPE_DOUBLE   FF_OPT_TYPE_DOUBLE
+#define AV_OPT_TYPE_FLOAT    FF_OPT_TYPE_FLOAT
+#define AV_OPT_TYPE_STRING   FF_OPT_TYPE_STRING
+#define AV_OPT_TYPE_RATIONAL FF_OPT_TYPE_RATIONAL
+#define AV_OPT_TYPE_BINARY   FF_OPT_TYPE_BINARY
+#define AV_OPT_TYPE_CONST    FF_OPT_TYPE_CONST
+#endif
+
 // A static flag used to determine if avformat has been initialised
 static int avformat_initialised = 0;
 
-// A locking mutex
-static pthread_mutex_t avformat_mutex;
-
-#if 0
-// These 3 functions should override the alloc functions in libavformat
-// but some formats or codecs seem to crash when used (wmv in particular)
-
-void *av_malloc( unsigned int size )
+static int avformat_lockmgr(void **mutex, enum AVLockOp op)
 {
-	return mlt_pool_alloc( size );
-}
+   pthread_mutex_t** pmutex = (pthread_mutex_t**) mutex;
 
-void *av_realloc( void *ptr, unsigned int size )
-{
-	return mlt_pool_realloc( ptr, size );
-}
+   switch (op)
+   {
+   case AV_LOCK_CREATE:
+      *pmutex = (pthread_mutex_t*) malloc(sizeof(pthread_mutex_t));
+       pthread_mutex_init(*pmutex, NULL);
+       break;
+   case AV_LOCK_OBTAIN:
+       pthread_mutex_lock(*pmutex);
+       break;
+   case AV_LOCK_RELEASE:
+       pthread_mutex_unlock(*pmutex);
+       break;
+   case AV_LOCK_DESTROY:
+       pthread_mutex_destroy(*pmutex);
+       free(*pmutex);
+       break;
+   }
 
-void av_free( void *ptr )
-{
-	return mlt_pool_release( ptr );
-}
-#endif
-
-void avformat_destroy( void *ignore )
-{
-	// Clean up
-	// av_free_static( ); -XXX this is deprecated
-
-	// Destroy the mutex
-	pthread_mutex_destroy( &avformat_mutex );
-}
-
-void avformat_lock( )
-{
-	// Lock the mutex now
-	pthread_mutex_lock( &avformat_mutex );
-}
-
-void avformat_unlock( )
-{
-	// Unlock the mutex now
-	pthread_mutex_unlock( &avformat_mutex );
+   return 0;
 }
 
 static void avformat_init( )
@@ -96,12 +89,14 @@ static void avformat_init( )
 	if ( avformat_initialised == 0 )
 	{
 		avformat_initialised = 1;
-		pthread_mutex_init( &avformat_mutex, NULL );
+		av_lockmgr_register( &avformat_lockmgr );
 		av_register_all( );
 #ifdef AVDEVICE
 		avdevice_register_all();
 #endif
-		mlt_factory_register_for_clean_up( NULL, avformat_destroy );
+#if LIBAVFORMAT_VERSION_INT >= ((53<<16)+(13<<8))
+		avformat_network_init();
+#endif
 		av_log_set_level( mlt_log_get_level() );
 	}
 }
@@ -125,8 +120,10 @@ static void *create_service( mlt_profile profile, mlt_service_type type, const c
 		return filter_avcolour_space_init( arg );
 	if ( !strcmp( id, "avdeinterlace" ) )
 		return filter_avdeinterlace_init( arg );
+#if LIBAVCODEC_VERSION_INT < ((54<<16)+(26<<8)+0)
 	if ( !strcmp( id, "avresample" ) )
 		return filter_avresample_init( arg );
+#endif
 #ifdef SWSCALE
 	if ( !strcmp( id, "swscale" ) )
 		return filter_swscale_init( profile, arg );
@@ -140,24 +137,28 @@ static void add_parameters( mlt_properties params, void *object, int req_flags,
 	const AVOption *opt = NULL;
 
 	// For each AVOption on the AVClass object
+#if LIBAVUTIL_VERSION_INT >= ((51<<16)+(12<<8)+0)
+	while ( ( opt = av_opt_next( object, opt ) ) )
+#else
 	while ( ( opt = av_next_option( object, opt ) ) )
+#endif
 	{
 		// If matches flags and not a binary option (not supported by Mlt)
-		if ( !( opt->flags & req_flags ) || ( opt->type == FF_OPT_TYPE_BINARY ) )
+		if ( !( opt->flags & req_flags ) || ( opt->type == AV_OPT_TYPE_BINARY ) )
             continue;
 
 		// Ignore constants (keyword values)
-		if ( !unit && opt->type == FF_OPT_TYPE_CONST )
+		if ( !unit && opt->type == AV_OPT_TYPE_CONST )
 			continue;
 		// When processing a groups of options (unit)...
 		// ...ignore non-constants
-		else if ( unit && opt->type != FF_OPT_TYPE_CONST )
+		else if ( unit && opt->type != AV_OPT_TYPE_CONST )
 			continue;
 		// ...ignore constants not in this group
-		else if ( unit && opt->type == FF_OPT_TYPE_CONST && strcmp( unit, opt->unit ) )
+		else if ( unit && opt->type == AV_OPT_TYPE_CONST && strcmp( unit, opt->unit ) )
 			continue;
 		// ..add constants to the 'values' sequence
-		else if ( unit && opt->type == FF_OPT_TYPE_CONST )
+		else if ( unit && opt->type == AV_OPT_TYPE_CONST )
 		{
 			char key[20];
 			snprintf( key, 20, "%d", mlt_properties_count( params ) );
@@ -192,11 +193,11 @@ static void add_parameters( mlt_properties params, void *object, int req_flags,
 
         switch ( opt->type )
 		{
-		case FF_OPT_TYPE_FLAGS:
+		case AV_OPT_TYPE_FLAGS:
 			mlt_properties_set( p, "type", "string" );
 			mlt_properties_set( p, "format", "flags" );
 			break;
-		case FF_OPT_TYPE_INT:
+		case AV_OPT_TYPE_INT:
 			if ( !opt->unit )
 			{
 				mlt_properties_set( p, "type", "integer" );
@@ -214,7 +215,7 @@ static void add_parameters( mlt_properties params, void *object, int req_flags,
 				mlt_properties_set( p, "format", "integer or keyword" );
 			}
 			break;
-		case FF_OPT_TYPE_INT64:
+		case AV_OPT_TYPE_INT64:
 			mlt_properties_set( p, "type", "integer" );
 			mlt_properties_set( p, "format", "64-bit" );
 			if ( opt->min != INT64_MIN )
@@ -225,7 +226,7 @@ static void add_parameters( mlt_properties params, void *object, int req_flags,
 			mlt_properties_set_int64( p, "default", (int64_t) opt->default_val.dbl );
 #endif
 			break;
-		case FF_OPT_TYPE_FLOAT:
+		case AV_OPT_TYPE_FLOAT:
 			mlt_properties_set( p, "type", "float" );
 			if ( opt->min != FLT_MIN && opt->min != -340282346638528859811704183484516925440.0 )
 				mlt_properties_set_double( p, "minimum", opt->min );
@@ -235,7 +236,7 @@ static void add_parameters( mlt_properties params, void *object, int req_flags,
 			mlt_properties_set_double( p, "default", opt->default_val.dbl );
 #endif
 			break;
-		case FF_OPT_TYPE_DOUBLE:
+		case AV_OPT_TYPE_DOUBLE:
 			mlt_properties_set( p, "type", "float" );
 			mlt_properties_set( p, "format", "double" );
 			if ( opt->min != DBL_MIN )
@@ -246,24 +247,24 @@ static void add_parameters( mlt_properties params, void *object, int req_flags,
 			mlt_properties_set_double( p, "default", opt->default_val.dbl );
 #endif
 			break;
-		case FF_OPT_TYPE_STRING:
+		case AV_OPT_TYPE_STRING:
 			mlt_properties_set( p, "type", "string" );
 #if LIBAVUTIL_VERSION_MAJOR > 50
 			mlt_properties_set( p, "default", opt->default_val.str );
 #endif
 			break;
-		case FF_OPT_TYPE_RATIONAL:
+		case AV_OPT_TYPE_RATIONAL:
 			mlt_properties_set( p, "type", "string" );
 			mlt_properties_set( p, "format", "numerator:denominator" );
 			break;
-		case FF_OPT_TYPE_CONST:
+		case AV_OPT_TYPE_CONST:
 		default:
 			mlt_properties_set( p, "type", "integer" );
 			mlt_properties_set( p, "format", "constant" );
 			break;
         }
 		// If the option belongs to a group (unit) and is not a constant (keyword value)
-		if ( opt->unit && opt->type != FF_OPT_TYPE_CONST )
+		if ( opt->unit && opt->type != AV_OPT_TYPE_CONST )
 		{
 			// Create a 'values' sequence.
 			mlt_properties values = mlt_properties_new();
@@ -310,7 +311,11 @@ static mlt_properties avformat_metadata( mlt_service_type type, const char *id,
 		// Annotate the yaml properties with AVOptions.
 		mlt_properties params = (mlt_properties) mlt_properties_get_data( result, "parameters", NULL );
 		AVFormatContext *avformat = avformat_alloc_context();
+#if LIBAVCODEC_VERSION_INT > ((53<<16)+(8<<8)+0)
+		AVCodecContext *avcodec = avcodec_alloc_context3( NULL );
+#else
 		AVCodecContext *avcodec = avcodec_alloc_context();
+#endif
 		int flags = ( type == consumer_type )? AV_OPT_FLAG_ENCODING_PARAM : AV_OPT_FLAG_DECODING_PARAM;
 
 		add_parameters( params, avformat, flags, NULL, NULL );
@@ -359,7 +364,9 @@ MLT_REPOSITORY
 	MLT_REGISTER( filter_type, "avcolour_space", create_service );
 	MLT_REGISTER( filter_type, "avcolor_space", create_service );
 	MLT_REGISTER( filter_type, "avdeinterlace", create_service );
+#if LIBAVCODEC_VERSION_INT < ((54<<16)+(26<<8)+0)
 	MLT_REGISTER( filter_type, "avresample", create_service );
+#endif
 #ifdef SWSCALE
 	MLT_REGISTER( filter_type, "swscale", create_service );
 #endif
