@@ -38,6 +38,7 @@
 #if LIBAVUTIL_VERSION_INT >= ((50<<16)+(38<<8)+0)
 #  include <libavutil/samplefmt.h>
 #else
+#  define AV_SAMPLE_FMT_U8  SAMPLE_FMT_U8
 #  define AV_SAMPLE_FMT_S16 SAMPLE_FMT_S16
 #  define AV_SAMPLE_FMT_S32 SAMPLE_FMT_S32
 #  define AV_SAMPLE_FMT_FLT SAMPLE_FMT_FLT
@@ -79,7 +80,7 @@ const char *avcodec_get_sample_fmt_name(int sample_fmt);
 #define POSITION_INITIAL (-2)
 #define POSITION_INVALID (-1)
 
-#define MAX_AUDIO_STREAMS (10)
+#define MAX_AUDIO_STREAMS (32)
 #define MAX_VDPAU_SURFACES (10)
 
 struct producer_avformat_s
@@ -91,6 +92,7 @@ struct producer_avformat_s
 	AVCodecContext *audio_codec[ MAX_AUDIO_STREAMS ];
 	AVCodecContext *video_codec;
 	AVFrame *av_frame;
+	AVPacket pkt;
 	ReSampleContext *audio_resample[ MAX_AUDIO_STREAMS ];
 	mlt_position audio_expected;
 	mlt_position video_expected;
@@ -116,12 +118,14 @@ struct producer_avformat_s
 	double resample_factor;
 	mlt_cache image_cache;
 	int colorspace;
+	int full_luma;
 	pthread_mutex_t video_mutex;
 	pthread_mutex_t audio_mutex;
 	mlt_deque apackets;
 	mlt_deque vpackets;
 	pthread_mutex_t packets_mutex;
 	pthread_mutex_t open_mutex;
+	int is_mutex_init;
 #ifdef VDPAU
 	struct
 	{
@@ -153,6 +157,7 @@ static void producer_set_up_audio( producer_avformat self, mlt_frame frame );
 static void apply_properties( void *obj, mlt_properties properties, int flags );
 static int video_codec_init( producer_avformat self, int index, mlt_properties properties );
 static void get_audio_streams_info( producer_avformat self );
+static mlt_audio_format pick_audio_format( int sample_fmt );
 
 #ifdef VDPAU
 #include "vdpau.c"
@@ -192,16 +197,6 @@ mlt_producer producer_avformat_init( mlt_profile profile, const char *service, c
 			// Register our get_frame implementation
 			producer->get_frame = producer_get_frame;
 
-			// init mutexes
-			pthread_mutex_init( &self->audio_mutex, NULL );
-			pthread_mutex_init( &self->video_mutex, NULL );
-			pthread_mutex_init( &self->packets_mutex, NULL );
-			pthread_mutex_init( &self->open_mutex, NULL );
-
-			// init queues
-			self->apackets = mlt_deque_init();
-			self->vpackets = mlt_deque_init();
-
 			if ( strcmp( service, "avformat-novalidate" ) )
 			{
 				// Open the file
@@ -283,6 +278,23 @@ int list_components( char* file )
 	return skip;
 }
 
+static int first_video_index( producer_avformat self )
+{
+	AVFormatContext *context = self->video_format? self->video_format : self->audio_format;
+	int i = -1; // not found
+
+	if ( context ) {
+		for ( i = 0; i < context->nb_streams; i++ ) {
+			if ( context->streams[i]->codec &&
+			     context->streams[i]->codec->codec_type == CODEC_TYPE_VIDEO )
+				break;
+		}
+		if ( i == context->nb_streams )
+			i = -1;
+	}
+	return i;
+}
+
 /** Find the default streams.
 */
 
@@ -379,8 +391,9 @@ static mlt_properties find_default_streams( producer_avformat self )
 				if ( !codec_context->channels )
 					break;
 				// Use first audio stream
-				if ( self->audio_index < 0 )
+				if ( self->audio_index < 0 && pick_audio_format( codec_context->sample_fmt ) != mlt_audio_none )
 					self->audio_index = i;
+
 				mlt_properties_set( meta_media, key, "audio" );
 #if LIBAVUTIL_VERSION_INT >= ((50<<16)+(38<<8)+0)
 				snprintf( key, sizeof(key), "meta.media.%d.codec.sample_fmt", i );
@@ -839,6 +852,14 @@ static int producer_open( producer_avformat self, mlt_profile profile, const cha
 	// Lock the service
 	if ( take_lock )
 	{
+		if ( !self->is_mutex_init )
+		{
+			pthread_mutex_init( &self->audio_mutex, NULL );
+			pthread_mutex_init( &self->video_mutex, NULL );
+			pthread_mutex_init( &self->packets_mutex, NULL );
+			pthread_mutex_init( &self->open_mutex, NULL );
+			self->is_mutex_init = 1;
+		}
 		pthread_mutex_lock( &self->audio_mutex );
 		pthread_mutex_lock( &self->video_mutex );
 	}
@@ -955,6 +976,11 @@ static int producer_open( producer_avformat self, mlt_profile profile, const cha
 	}
 	if ( filename )
 		free( filename );
+	if ( !error )
+	{
+		self->apackets = mlt_deque_init();
+		self->vpackets = mlt_deque_init();
+	}
 
 	if ( self->dummy_context )
 	{
@@ -1032,6 +1058,31 @@ static int64_t best_pts( producer_avformat self, int64_t pts, int64_t dts )
 		return dts;
 }
 
+static void find_first_pts( producer_avformat self, int video_index )
+{
+	// find initial PTS
+	AVFormatContext *context = self->video_format? self->video_format : self->audio_format;
+	int ret = 0;
+	int toscan = 500;
+	AVPacket pkt;
+
+	while ( ret >= 0 && toscan-- > 0 )
+	{
+		ret = av_read_frame( context, &pkt );
+		if ( ret >= 0 && pkt.stream_index == video_index && ( pkt.flags & PKT_FLAG_KEY ) )
+		{
+			mlt_log_debug( MLT_PRODUCER_SERVICE(self->parent),
+				"first_pts %"PRId64" dts %"PRId64" pts_dts_delta %d\n",
+				pkt.pts, pkt.dts, (int)(pkt.pts - pkt.dts) );
+			self->first_pts = best_pts( self, pkt.pts, pkt.dts );
+			if ( self->first_pts != AV_NOPTS_VALUE )
+				toscan = 0;
+		}
+		av_free_packet( &pkt );
+	}
+	av_seek_frame( context, -1, 0, AVSEEK_FLAG_BACKWARD );
+}
+
 static int seek_video( producer_avformat self, mlt_position position,
 	int64_t req_position, int preseek )
 {
@@ -1055,29 +1106,8 @@ static int seek_video( producer_avformat self, mlt_position position,
 		double source_fps = mlt_properties_get_double( properties, "meta.media.frame_rate_num" ) /
 			mlt_properties_get_double( properties, "meta.media.frame_rate_den" );
 
-		// find initial PTS
 		if ( self->last_position == POSITION_INITIAL )
-		{
-			int ret = 0;
-			int toscan = 500;
-			AVPacket pkt;
-
-			while ( ret >= 0 && toscan-- > 0 )
-			{
-				ret = av_read_frame( context, &pkt );
-				if ( ret >= 0 && pkt.stream_index == self->video_index && ( pkt.flags & PKT_FLAG_KEY ) )
-				{
-					mlt_log_debug( MLT_PRODUCER_SERVICE(producer),
-						"first_pts %"PRId64" dts %"PRId64" pts_dts_delta %d\n",
-						pkt.pts, pkt.dts, (int)(pkt.pts - pkt.dts) );
-					self->first_pts = best_pts( self, pkt.pts, pkt.dts );
-					if ( self->first_pts != AV_NOPTS_VALUE )
-						toscan = 0;
-				}
-				av_free_packet( &pkt );
-			}
-			av_seek_frame( context, -1, 0, AVSEEK_FLAG_BACKWARD );
-		}
+			find_first_pts( self, self->video_index );
 
 		if ( self->av_frame && position + 1 == self->video_expected )
 		{
@@ -1218,7 +1248,7 @@ static void set_luma_transfer( struct SwsContext *context, int colorspace, int u
 #endif
 }
 
-static mlt_image_format pick_format( enum PixelFormat pix_fmt )
+static mlt_image_format pick_pix_format( enum PixelFormat pix_fmt )
 {
 	switch ( pix_fmt )
 	{
@@ -1244,11 +1274,39 @@ static mlt_image_format pick_format( enum PixelFormat pix_fmt )
 	}
 }
 
-static void convert_image( AVFrame *frame, uint8_t *buffer, int pix_fmt,
-	mlt_image_format *format, int width, int height, int colorspace, uint8_t **alpha )
+static mlt_audio_format pick_audio_format( int sample_fmt )
+{
+	switch ( sample_fmt )
+	{
+	// interleaved
+	case AV_SAMPLE_FMT_U8:
+		return mlt_audio_u8;
+	case AV_SAMPLE_FMT_S16:
+		return mlt_audio_s16;
+	case AV_SAMPLE_FMT_S32:
+		return mlt_audio_s32le;
+	case AV_SAMPLE_FMT_FLT:
+		return mlt_audio_f32le;
+	// planar - this producer converts planar to interleaved
+#if LIBAVUTIL_VERSION_INT >= ((51<<16)+(17<<8)+0)
+	case AV_SAMPLE_FMT_U8P:
+		return mlt_audio_u8;
+	case AV_SAMPLE_FMT_S16P:
+		return mlt_audio_s16;
+	case AV_SAMPLE_FMT_S32P:
+		return mlt_audio_s32le;
+	case AV_SAMPLE_FMT_FLTP:
+		return mlt_audio_f32le;
+#endif
+	default:
+		return mlt_audio_none;
+	}
+}
+
+static void convert_image( producer_avformat self, AVFrame *frame, uint8_t *buffer, int pix_fmt,
+	mlt_image_format *format, int width, int height, uint8_t **alpha )
 {
 #ifdef SWSCALE
-	int full_range = -1;
 	int flags = SWS_BICUBIC | SWS_ACCURATE_RND;
 
 #ifdef USE_MMX
@@ -1288,7 +1346,7 @@ static void convert_image( AVFrame *frame, uint8_t *buffer, int pix_fmt,
 		output.linesize[0] = width;
 		output.linesize[1] = width >> 1;
 		output.linesize[2] = width >> 1;
-		set_luma_transfer( context, colorspace, full_range );
+		set_luma_transfer( context, self->colorspace, -1 );
 		sws_scale( context, (const uint8_t* const*) frame->data, frame->linesize, 0, height,
 			output.data, output.linesize);
 		sws_freeContext( context );
@@ -1299,7 +1357,7 @@ static void convert_image( AVFrame *frame, uint8_t *buffer, int pix_fmt,
 			width, height, PIX_FMT_RGB24, flags | SWS_FULL_CHR_H_INT, NULL, NULL, NULL);
 		AVPicture output;
 		avpicture_fill( &output, buffer, PIX_FMT_RGB24, width, height );
-		set_luma_transfer( context, colorspace, full_range );
+		set_luma_transfer( context, self->colorspace, self->full_luma );
 		sws_scale( context, (const uint8_t* const*) frame->data, frame->linesize, 0, height,
 			output.data, output.linesize);
 		sws_freeContext( context );
@@ -1310,7 +1368,7 @@ static void convert_image( AVFrame *frame, uint8_t *buffer, int pix_fmt,
 			width, height, PIX_FMT_RGBA, flags | SWS_FULL_CHR_H_INT, NULL, NULL, NULL);
 		AVPicture output;
 		avpicture_fill( &output, buffer, PIX_FMT_RGBA, width, height );
-		set_luma_transfer( context, colorspace, full_range );
+		set_luma_transfer( context, self->colorspace, self->full_luma );
 		sws_scale( context, (const uint8_t* const*) frame->data, frame->linesize, 0, height,
 			output.data, output.linesize);
 		sws_freeContext( context );
@@ -1321,7 +1379,7 @@ static void convert_image( AVFrame *frame, uint8_t *buffer, int pix_fmt,
 			width, height, PIX_FMT_YUYV422, flags | SWS_FULL_CHR_H_INP, NULL, NULL, NULL);
 		AVPicture output;
 		avpicture_fill( &output, buffer, PIX_FMT_YUYV422, width, height );
-		set_luma_transfer( context, colorspace, full_range );
+		set_luma_transfer( context, self->colorspace, -1 );
 		sws_scale( context, (const uint8_t* const*) frame->data, frame->linesize, 0, height,
 			output.data, output.linesize);
 		sws_freeContext( context );
@@ -1344,7 +1402,7 @@ static void convert_image( AVFrame *frame, uint8_t *buffer, int pix_fmt,
 		avpicture_fill( &output, buffer, PIX_FMT_RGB24, width, height );
 		img_convert( &output, PIX_FMT_RGB24, (AVPicture *)frame, pix_fmt, width, height );
 	}
-	else if ( format == mlt_image_rgb24a || format == mlt_image_opengl )
+	else if ( *format == mlt_image_rgb24a || *format == mlt_image_opengl )
 	{
 		AVPicture output;
 		avpicture_fill( &output, buffer, PIX_FMT_RGB32, width, height );
@@ -1393,7 +1451,7 @@ static int producer_get_image( mlt_frame frame, uint8_t **buffer, mlt_image_form
 	mlt_properties frame_properties = MLT_FRAME_PROPERTIES( frame );
 
 	// Obtain the frame number of this frame
-	mlt_position position = mlt_properties_get_position( frame_properties, "avformat_position" );
+	mlt_position position = mlt_frame_original_position( frame );
 
 	// Get the producer properties
 	mlt_properties properties = MLT_PRODUCER_PROPERTIES( producer );
@@ -1437,7 +1495,7 @@ static int producer_get_image( mlt_frame frame, uint8_t **buffer, mlt_image_form
 	}
 	if ( self->image_cache )
 	{
-		mlt_frame original = mlt_cache_get_frame( self->image_cache, mlt_frame_get_position( frame ) );
+		mlt_frame original = mlt_cache_get_frame( self->image_cache, position );
 		if ( original )
 		{
 			mlt_properties orig_props = MLT_FRAME_PROPERTIES( original );
@@ -1465,9 +1523,6 @@ static int producer_get_image( mlt_frame frame, uint8_t **buffer, mlt_image_form
 	}
 	// Cache miss
 
-	// Packet
-	AVPacket pkt;
-
 	// We may want to use the source fps if available
 	double source_fps = mlt_properties_get_double( properties, "meta.media.frame_rate_num" ) /
 		mlt_properties_get_double( properties, "meta.media.frame_rate_den" );
@@ -1504,7 +1559,7 @@ static int producer_get_image( mlt_frame frame, uint8_t **buffer, mlt_image_form
 			codec_context->pix_fmt == PIX_FMT_RGBA ||
 			codec_context->pix_fmt == PIX_FMT_ABGR ||
 			codec_context->pix_fmt == PIX_FMT_BGRA )
-		*format = pick_format( codec_context->pix_fmt );
+		*format = pick_pix_format( codec_context->pix_fmt );
 
 	// Duplicate the last image if necessary
 	if ( self->av_frame && self->av_frame->linesize[0]
@@ -1526,13 +1581,13 @@ static int producer_get_image( mlt_frame frame, uint8_t **buffer, mlt_image_form
 				picture.linesize[0] = codec_context->width;
 				picture.linesize[1] = codec_context->width / 2;
 				picture.linesize[2] = codec_context->width / 2;
-				convert_image( (AVFrame*) &picture, *buffer,
-					PIX_FMT_YUV420P, format, *width, *height, self->colorspace, &alpha );
+				convert_image( self, (AVFrame*) &picture, *buffer,
+					PIX_FMT_YUV420P, format, *width, *height, &alpha );
 			}
 			else
 #endif
-			convert_image( self->av_frame, *buffer, codec_context->pix_fmt,
-				format, *width, *height, self->colorspace, &alpha );
+			convert_image( self, self->av_frame, *buffer, codec_context->pix_fmt,
+				format, *width, *height, &alpha );
 			got_picture = 1;
 		}
 	}
@@ -1542,8 +1597,6 @@ static int producer_get_image( mlt_frame frame, uint8_t **buffer, mlt_image_form
 		int64_t int_position = 0;
 		int decode_errors = 0;
 
-		av_init_packet( &pkt );
-
 		// Construct an AVFrame for YUV422 conversion
 		if ( !self->av_frame )
 			self->av_frame = avcodec_alloc_frame( );
@@ -1551,22 +1604,25 @@ static int producer_get_image( mlt_frame frame, uint8_t **buffer, mlt_image_form
 		while( ret >= 0 && !got_picture )
 		{
 			// Read a packet
+			if ( self->pkt.stream_index == self->video_index )
+				av_free_packet( &self->pkt );
+			av_init_packet( &self->pkt );
 			pthread_mutex_lock( &self->packets_mutex );
 			if ( mlt_deque_count( self->vpackets ) )
 			{
 				AVPacket *tmp = (AVPacket*) mlt_deque_pop_front( self->vpackets );
-				pkt = *tmp;
+				self->pkt = *tmp;
 				free( tmp );
 			}
 			else
 			{
-				ret = av_read_frame( context, &pkt );
-				if ( ret >= 0 && !self->seekable && pkt.stream_index == self->audio_index )
+				ret = av_read_frame( context, &self->pkt );
+				if ( ret >= 0 && !self->seekable && self->pkt.stream_index == self->audio_index )
 				{
-					if ( !av_dup_packet( &pkt ) )
+					if ( !av_dup_packet( &self->pkt ) )
 					{
 						AVPacket *tmp = malloc( sizeof(AVPacket) );
-						*tmp = pkt;
+						*tmp = self->pkt;
 						mlt_deque_push_back( self->apackets, tmp );
 					}
 				}
@@ -1574,9 +1630,9 @@ static int producer_get_image( mlt_frame frame, uint8_t **buffer, mlt_image_form
 			pthread_mutex_unlock( &self->packets_mutex );
 
 			// We only deal with video from the selected video_index
-			if ( ret >= 0 && pkt.stream_index == self->video_index && pkt.size > 0 )
+			if ( ret >= 0 && self->pkt.stream_index == self->video_index && self->pkt.size > 0 )
 			{
-				int64_t pts = best_pts( self, pkt.pts, pkt.dts );
+				int64_t pts = best_pts( self, self->pkt.pts, self->pkt.dts );
 				if ( pts != AV_NOPTS_VALUE )
 				{
 					if ( !self->seekable && self->first_pts == AV_NOPTS_VALUE )
@@ -1591,7 +1647,7 @@ static int producer_get_image( mlt_frame frame, uint8_t **buffer, mlt_image_form
 				}
 				mlt_log_debug( MLT_PRODUCER_SERVICE(producer),
 					"V pkt.pts %"PRId64" pkt.dts %"PRId64" req_pos %"PRId64" cur_pos %"PRId64" pkt_pos %"PRId64"\n",
-					pkt.pts, pkt.dts, req_position, self->current_position, int_position );
+					self->pkt.pts, self->pkt.dts, req_position, self->current_position, int_position );
 
 				// Make a dumb assumption on streams that contain wild timestamps
 				if ( abs( req_position - int_position ) > 999 )
@@ -1618,9 +1674,9 @@ static int producer_get_image( mlt_frame frame, uint8_t **buffer, mlt_image_form
 					if ( int_position >= req_position )
 						codec_context->skip_loop_filter = AVDISCARD_NONE;
 #if (LIBAVCODEC_VERSION_INT >= ((52<<16)+(26<<8)+0))
-					ret = avcodec_decode_video2( codec_context, self->av_frame, &got_picture, &pkt );
+					ret = avcodec_decode_video2( codec_context, self->av_frame, &got_picture, &self->pkt );
 #else
-					ret = avcodec_decode_video( codec_context, self->av_frame, &got_picture, pkt.data, pkt.size );
+					ret = avcodec_decode_video( codec_context, self->av_frame, &got_picture, self->pkt.data, self->pkt.size );
 #endif
 					// Note: decode may fail at the beginning of MPEGfile (B-frames referencing before first I-frame), so allow a few errors.
 					if ( ret < 0 )
@@ -1655,7 +1711,7 @@ static int producer_get_image( mlt_frame frame, uint8_t **buffer, mlt_image_form
 					else if ( int_position >= req_position )
 						codec_context->skip_loop_filter = AVDISCARD_NONE;
 				}
-				mlt_log_debug( MLT_PRODUCER_SERVICE(producer), " got_pic %d key %d\n", got_picture, pkt.flags & PKT_FLAG_KEY );
+				mlt_log_debug( MLT_PRODUCER_SERVICE(producer), " got_pic %d key %d\n", got_picture, self->pkt.flags & PKT_FLAG_KEY );
 			}
 
 			// Now handle the picture if we have one
@@ -1688,8 +1744,8 @@ static int producer_get_image( mlt_frame frame, uint8_t **buffer, mlt_image_form
 							VdpStatus status = vdp_surface_get_bits( render->surface, dest_format, planes, pitches );
 							if ( status == VDP_STATUS_OK )
 							{
-								convert_image( self->av_frame, *buffer, PIX_FMT_YUV420P,
-									format, *width, *height, self->colorspace, &alpha );
+								convert_image( self, self->av_frame, *buffer, PIX_FMT_YUV420P,
+									format, *width, *height, &alpha );
 							}
 							else
 							{
@@ -1705,8 +1761,8 @@ static int producer_get_image( mlt_frame frame, uint8_t **buffer, mlt_image_form
 					}
 					else
 #endif
-					convert_image( self->av_frame, *buffer, codec_context->pix_fmt,
-						format, *width, *height, self->colorspace, &alpha );
+					convert_image( self, self->av_frame, *buffer, codec_context->pix_fmt,
+						format, *width, *height, &alpha );
 					self->top_field_first |= self->av_frame->top_field_first;
 					self->current_position = int_position;
 				}
@@ -1715,8 +1771,11 @@ static int producer_get_image( mlt_frame frame, uint8_t **buffer, mlt_image_form
 					got_picture = 0;
 				}
 			}
-			if ( self->seekable || pkt.stream_index != self->audio_index )
-				av_free_packet( &pkt );
+
+			// Free packet data if not video and not live audio packet
+			if ( self->pkt.stream_index != self->video_index &&
+				 !( !self->seekable && self->pkt.stream_index == self->audio_index ) )
+				av_free_packet( &self->pkt );
 		}
 	}
 
@@ -1751,13 +1810,13 @@ static int producer_get_image( mlt_frame frame, uint8_t **buffer, mlt_image_form
 				picture.linesize[0] = codec_context->width;
 				picture.linesize[1] = codec_context->width / 2;
 				picture.linesize[2] = codec_context->width / 2;
-				convert_image( (AVFrame*) &picture, *buffer,
-					PIX_FMT_YUV420P, format, *width, *height, self->colorspace, &alpha );
+				convert_image( self, (AVFrame*) &picture, *buffer,
+					PIX_FMT_YUV420P, format, *width, *height, &alpha );
 			}
 			else
 #endif
-			convert_image( self->av_frame, *buffer, codec_context->pix_fmt,
-				format, *width, *height, self->colorspace, &alpha );
+			convert_image( self, self->av_frame, *buffer, codec_context->pix_fmt,
+				format, *width, *height, &alpha );
 			got_picture = 1;
 		}
 	}
@@ -1787,6 +1846,11 @@ exit_get_image:
 	mlt_properties_set_int( properties, "meta.media.progressive", mlt_properties_get_int( frame_properties, "progressive" ) );
 	mlt_service_unlock( MLT_PRODUCER_SERVICE( producer ) );
 
+	// If we already have RGB, then the full range processing either happened already
+	// or does not apply (RGB source).
+	if ( *format == mlt_image_rgb24 || *format == mlt_image_rgb24a || *format == mlt_image_opengl )
+		mlt_properties_set( frame_properties, "force_full_luma", NULL );
+
 	return !got_picture;
 }
 
@@ -1902,14 +1966,6 @@ static int video_codec_init( producer_avformat self, int index, mlt_properties p
 		double source_fps = (double) self->video_codec->time_base.den /
 								   ( self->video_codec->time_base.num == 0 ? 1 : self->video_codec->time_base.num );
 		
-		if ( mlt_properties_get( properties, "force_fps" ) )
-		{
-			source_fps = mlt_properties_get_double( properties, "force_fps" );
-			stream->time_base = av_d2q( source_fps, 1024 );
-			mlt_properties_set_int( properties, "meta.media.frame_rate_num", stream->time_base.num );
-			mlt_properties_set_int( properties, "meta.media.frame_rate_den", stream->time_base.den );
-		}
-		else
 		{
 			// If the muxer reports a frame rate different than the codec
 #if LIBAVFORMAT_VERSION_INT >= ((52<<16)+(42<<8)+0)
@@ -1949,6 +2005,15 @@ static int video_codec_init( producer_avformat self, int index, mlt_properties p
 				mlt_properties_set_int( properties, "meta.media.frame_rate_den", frame_rate.den );
 			}
 		}
+		if ( mlt_properties_get( properties, "force_fps" ) )
+		{
+			double source_fps = mlt_properties_get_double( properties, "force_fps" );
+			AVRational fps = av_d2q( source_fps, 1024 );
+			stream->time_base.num *= mlt_properties_get_int( properties, "meta.media.frame_rate_num" ) * fps.den;
+			stream->time_base.den *= mlt_properties_get_int( properties, "meta.media.frame_rate_den" ) * fps.num;
+			mlt_properties_set_int( properties, "meta.media.frame_rate_num", fps.num );
+			mlt_properties_set_int( properties, "meta.media.frame_rate_den", fps.den );
+		}
 
 		// Set the YUV colorspace from override or detect
 		self->colorspace = mlt_properties_get_int( properties, "force_colorspace" );
@@ -1976,6 +2041,15 @@ static int video_codec_init( producer_avformat self, int index, mlt_properties p
 #endif
 		// Let apps get chosen colorspace
 		mlt_properties_set_int( properties, "meta.media.colorspace", self->colorspace );
+
+		self->full_luma = -1;
+#if LIBAVCODEC_VERSION_INT >= ((52<<16)+(72<<8)+2)
+		mlt_log_debug( MLT_PRODUCER_SERVICE(self->parent), "color_range %d\n", codec_context->color_range );
+		if ( codec_context->color_range == AVCOL_RANGE_JPEG )
+			self->full_luma = 1;
+#endif
+		if ( mlt_properties_get( properties, "set.force_full_luma" ) )
+			self->full_luma = mlt_properties_get_int( properties, "set.force_full_luma" );
 	}
 	return self->video_codec && self->video_index > -1;
 }
@@ -2074,8 +2148,17 @@ static int seek_audio( producer_avformat self, mlt_position position, double tim
 	int paused = 0;
 
 	// Seek if necessary
-	if ( self->seekable && position != self->audio_expected )
+	if ( self->seekable && ( position != self->audio_expected || self->last_position < 0 ) )
 	{
+		if ( self->last_position == POSITION_INITIAL )
+		{
+			int video_index = self->video_index;
+			if ( video_index == -1 )
+				video_index = first_video_index( self );
+			if ( video_index >= 0 )
+				find_first_pts( self, video_index );
+		}
+
 		if ( position + 1 == self->audio_expected )
 		{
 			// We're paused - silence required
@@ -2114,6 +2197,19 @@ static int sample_bytes( AVCodecContext *context )
 #endif
 }
 
+static void planar_to_interleaved( uint8_t *dest, uint8_t *src, int samples, int channels, int bytes_per_sample )
+{
+	int s, c;
+	for ( s = 0; s < samples; s++ )
+	{
+		for ( c = 0; c < channels; c++ )
+		{
+			memcpy( dest, src + ( c * samples + s ) * bytes_per_sample, bytes_per_sample );
+			dest += bytes_per_sample;
+		}
+	}
+}
+
 static int decode_audio( producer_avformat self, int *ignore, AVPacket pkt, int channels, int samples, double timecode, double fps )
 {
 	// Fetch the audio_format
@@ -2181,8 +2277,22 @@ static int decode_audio( producer_avformat self, int *ignore, AVPacket pkt, int
 			}
 			else
 			{
-				// Straight copy to audio buffer
-				memcpy( &audio_buffer[ audio_used * codec_context->channels * sizeof_sample ], decode_buffer, data_size );
+				uint8_t *source = decode_buffer;
+				uint8_t *dest = &audio_buffer[ audio_used * codec_context->channels * sizeof_sample ];
+				switch ( codec_context->sample_fmt )
+				{
+#if LIBAVUTIL_VERSION_INT >= ((51<<16)+(17<<8)+0)
+				case AV_SAMPLE_FMT_U8P:
+				case AV_SAMPLE_FMT_S16P:
+				case AV_SAMPLE_FMT_S32P:
+				case AV_SAMPLE_FMT_FLTP:
+					planar_to_interleaved( dest, source, convert_samples, codec_context->channels, sizeof_sample );
+					break;
+#endif
+				default:
+					// Straight copy to audio buffer
+					memcpy( dest, decode_buffer, data_size );
+				}
 				audio_used += convert_samples;
 			}
 
@@ -2202,7 +2312,7 @@ static int decode_audio( producer_avformat self, int *ignore, AVPacket pkt, int
 	if ( pkt.pts >= 0 && ( self->seekable || self->video_format ) && *ignore == 0 && audio_used > samples / 2 )
 	{
 		int64_t pts = pkt.pts;
-		if ( self->first_pts != 0 )
+		if ( self->first_pts != AV_NOPTS_VALUE )
 			pts -= self->first_pts;
 		else if ( context->start_time != AV_NOPTS_VALUE )
 			pts -= context->start_time;
@@ -2223,6 +2333,9 @@ static int decode_audio( producer_avformat self, int *ignore, AVPacket pkt, int
 				// We are ahead, so seek backwards some more
 				seek_audio( self, req_position, timecode - 1.0 );
 		}
+		// Cancel the find_first_pts() in seek_audio()
+		if ( self->video_index == -1 && self->last_position == POSITION_INITIAL )
+			self->last_position = int_position;
 	}
 
 	self->audio_used[ index ] = audio_used;
@@ -2240,7 +2353,7 @@ static int producer_get_audio( mlt_frame frame, void **buffer, mlt_audio_format
 	pthread_mutex_lock( &self->audio_mutex );
 	
 	// Obtain the frame number of this frame
-	mlt_position position = mlt_properties_get_position( MLT_FRAME_PROPERTIES( frame ), "avformat_position" );
+	mlt_position position = mlt_frame_original_position( frame );
 
 	// Calculate the real time code
 	double real_timecode = producer_time_of_frame( self->parent, position );
@@ -2270,7 +2383,7 @@ static int producer_get_audio( mlt_frame frame, void **buffer, mlt_audio_format
 	if ( self->audio_index == INT_MAX )
 	{
 		index = 0;
-		index_max = context->nb_streams;
+		index_max = FFMIN( MAX_AUDIO_STREAMS, context->nb_streams );
 		*channels = self->total_channels;
 		*samples = *samples * FFMAX( self->max_frequency, *frequency ) / *frequency;
 		*frequency = FFMAX( self->max_frequency, *frequency );
@@ -2284,6 +2397,7 @@ static int producer_get_audio( mlt_frame frame, void **buffer, mlt_audio_format
 
 		if ( codec_context && !self->audio_buffer[ index ] )
 		{
+#if LIBAVCODEC_VERSION_INT < ((54<<16)+(26<<8)+0)
 			// Check for resample and create if necessary
 			if ( codec_context->channels <= 2 )
 			{
@@ -2306,6 +2420,7 @@ static int producer_get_audio( mlt_frame frame, void **buffer, mlt_audio_format
 #endif
 			}
 			else
+#endif
 			{
 				codec_context->request_channels = self->audio_index == INT_MAX ? codec_context->channels : *channels;
 				sizeof_sample = sample_bytes( codec_context );
@@ -2321,7 +2436,7 @@ static int producer_get_audio( mlt_frame frame, void **buffer, mlt_audio_format
 	}
 
 	// Get the audio if required
-	if ( !paused )
+	if ( !paused && *frequency > 0 )
 	{
 		int ret	= 0;
 		int got_audio = 0;
@@ -2349,7 +2464,7 @@ static int producer_get_audio( mlt_frame frame, void **buffer, mlt_audio_format
 			{
 				// Check if there is enough audio for all streams
 				got_audio = 1;
-				for ( index = 0; got_audio && index < context->nb_streams; index++ )
+				for ( index = 0; got_audio && index < index_max; index++ )
 					if ( ( self->audio_codec[ index ] && self->audio_used[ index ] < *samples ) || ignore[ index ] )
 						got_audio = 0;
 				if ( got_audio )
@@ -2401,20 +2516,16 @@ static int producer_get_audio( mlt_frame frame, void **buffer, mlt_audio_format
 			index = self->audio_index;
 			*channels = self->audio_codec[ index ]->channels;
 			*frequency = self->audio_codec[ index ]->sample_rate;
-			*format = self->audio_codec[ index ]->sample_fmt == AV_SAMPLE_FMT_S32 ? mlt_audio_s32le
-				: self->audio_codec[ index ]->sample_fmt == AV_SAMPLE_FMT_FLT ? mlt_audio_f32le
-				: mlt_audio_s16;
+			*format = pick_audio_format( self->audio_codec[ index ]->sample_fmt );
 			sizeof_sample = sample_bytes( self->audio_codec[ index ] );
 		}
 		else if ( self->audio_index == INT_MAX )
 		{
-			// This only works if all audio tracks have the same sample format.
 			for ( index = 0; index < index_max; index++ )
 				if ( self->audio_codec[ index ] && !self->audio_resample[ index ] )
 				{
-					*format = self->audio_codec[ index ]->sample_fmt == AV_SAMPLE_FMT_S32 ? mlt_audio_s32le
-						: self->audio_codec[ index ]->sample_fmt == AV_SAMPLE_FMT_FLT ? mlt_audio_f32le
-						: mlt_audio_s16;
+					// XXX: This only works if all audio tracks have the same sample format.
+					*format = pick_audio_format( self->audio_codec[ index ]->sample_fmt );
 					sizeof_sample = sample_bytes( self->audio_codec[ index ] );
 					break;
 				}
@@ -2582,6 +2693,11 @@ static void producer_set_up_audio( producer_avformat self, mlt_frame frame )
 		index = self->audio_index;
 		mlt_properties_set_int( properties, "audio_index", index );
 	}
+	if ( context && index > -1 && index < INT_MAX &&
+		 pick_audio_format( context->streams[ index ]->codec->sample_fmt ) == mlt_audio_none )
+	{
+		index = -1;
+	}
 
 	// Update the audio properties if the index changed
 	if ( context && index > -1 && index != self->audio_index )
@@ -2613,8 +2729,8 @@ static void producer_set_up_audio( producer_avformat self, mlt_frame frame )
 		// Set the frame properties
 		if ( index < MAX_AUDIO_STREAMS )
 		{
-			mlt_properties_set_int( frame_properties, "frequency", self->audio_codec[ index ]->sample_rate );
-			mlt_properties_set_int( frame_properties, "channels", self->audio_codec[ index ]->channels );
+			mlt_properties_set_int( frame_properties, "audio_frequency", self->audio_codec[ index ]->sample_rate );
+			mlt_properties_set_int( frame_properties, "audio_channels", self->audio_codec[ index ]->channels );
 		}
 	}
 	if ( context && index > -1 )
@@ -2669,7 +2785,7 @@ static int producer_get_frame( mlt_producer producer, mlt_frame_ptr frame, int i
 
 	// Set the position of this producer
 	mlt_position position = self->seekable ? mlt_producer_frame( producer ) : self->nonseek_position++;
-	mlt_properties_set_position( MLT_FRAME_PROPERTIES( *frame ), "avformat_position", position );
+	mlt_properties_set_position( MLT_FRAME_PROPERTIES( *frame ), "original_position", position );
 
 	// Calculate the next timecode
 	mlt_producer_prepare_next( producer );
@@ -2682,6 +2798,7 @@ static void producer_avformat_close( producer_avformat self )
 	mlt_log_debug( NULL, "producer_avformat_close\n" );
 
 	// Cleanup av contexts
+	av_free_packet( &self->pkt );
 	av_free( self->av_frame );
 	pthread_mutex_lock( &self->open_mutex );
 	int i;
@@ -2722,10 +2839,13 @@ static void producer_avformat_close( producer_avformat self )
 		mlt_cache_close( self->image_cache );
 
 	// Cleanup the mutexes
-	pthread_mutex_destroy( &self->audio_mutex );
-	pthread_mutex_destroy( &self->video_mutex );
-	pthread_mutex_destroy( &self->packets_mutex );
-	pthread_mutex_destroy( &self->open_mutex );
+	if ( self->is_mutex_init )
+	{
+		pthread_mutex_destroy( &self->audio_mutex );
+		pthread_mutex_destroy( &self->video_mutex );
+		pthread_mutex_destroy( &self->packets_mutex );
+		pthread_mutex_destroy( &self->open_mutex );
+	}
 
 	// Cleanup the packet queues
 	AVPacket *pkt;
