@@ -32,12 +32,15 @@
 #include <math.h>
 
 
-#ifdef WIN32
+#if defined(WIN32)
 #define LIBSUF ".dll"
-#define FREI0R_PLUGIN_PATH "\\..\\..\\lib\\frei0r-1"
+#define FREI0R_PLUGIN_PATH "\\lib\\frei0r-1"
+#elif defined(__DARWIN__) && defined(RELOCATABLE)
+#define LIBSUF ".so"
+#define FREI0R_PLUGIN_PATH "/lib/frei0r-1"
 #else
 #define LIBSUF ".so"
-#define FREI0R_PLUGIN_PATH "lib/frei0r-1:/usr/lib/frei0r-1:/usr/lib64/frei0r-1:/opt/local/lib/frei0r-1:/usr/local/lib/frei0r-1:$HOME/.frei0r-1/lib"
+#define FREI0R_PLUGIN_PATH "/usr/lib/frei0r-1:/usr/lib64/frei0r-1:/opt/local/lib/frei0r-1:/usr/local/lib/frei0r-1:$HOME/.frei0r-1/lib"
 #endif
 
 #define GET_FREI0R_PATH (getenv("FREI0R_PATH") ? getenv("FREI0R_PATH") : getenv("MLT_FREI0R_PLUGIN_PATH") ? getenv("MLT_FREI0R_PLUGIN_PATH") : FREI0R_PLUGIN_PATH)
@@ -53,8 +56,13 @@ extern mlt_frame transition_process( mlt_transition transition, mlt_frame a_fram
 static char* get_frei0r_path()
 {
 #ifdef WIN32
-	char *dirname = malloc( strlen( mlt_environment( "MLT_DATA" ) ) + strlen( FREI0R_PLUGIN_PATH ) + 1 );
-	strcpy( dirname, mlt_environment( "MLT_DATA" ) );
+	char *dirname = malloc( strlen( mlt_environment( "MLT_APPDIR" ) ) + strlen( FREI0R_PLUGIN_PATH ) + 1 );
+	strcpy( dirname, mlt_environment( "MLT_APPDIR" ) );
+	strcat( dirname, FREI0R_PLUGIN_PATH );
+	return dirname;
+#elif defined(__DARWIN__) && defined(RELOCATABLE)
+	char *dirname = malloc( strlen( mlt_environment( "MLT_APPDIR" ) ) + strlen( FREI0R_PLUGIN_PATH ) + 1 );
+	strcpy( dirname, mlt_environment( "MLT_APPDIR" ) );
 	strcat( dirname, FREI0R_PLUGIN_PATH );
 	return dirname;
 #else
@@ -270,13 +278,13 @@ static void * load_lib( mlt_profile profile, mlt_service_type type , void* handl
 			}
 		}
 		check_thread_safe( properties, name );
-		mlt_properties_set_data(properties, "_dlclose_handle", handle , sizeof (void*) , NULL , NULL );
+		mlt_properties_set_data(properties, "_dlclose_handle", handle , sizeof ( handle ) , NULL , NULL );
 		mlt_properties_set_data(properties, "_dlclose", dlclose , sizeof (void*) , NULL , NULL );
-		mlt_properties_set_data(properties, "f0r_construct", f0r_construct , sizeof(void*),NULL,NULL);
-		mlt_properties_set_data(properties, "f0r_update", f0r_update , sizeof(void*),NULL,NULL);
+		mlt_properties_set_data(properties, "f0r_construct", f0r_construct , sizeof( f0r_construct ),NULL,NULL);
+		mlt_properties_set_data(properties, "f0r_update", f0r_update , sizeof( f0r_update ),NULL,NULL);
 		if (f0r_update2)
-			mlt_properties_set_data(properties, "f0r_update2", f0r_update2 , sizeof(void*),NULL,NULL);
-		mlt_properties_set_data(properties, "f0r_destruct", f0r_destruct , sizeof(void*),NULL,NULL);
+			mlt_properties_set_data(properties, "f0r_update2", f0r_update2 , sizeof( f0r_update2 ),NULL,NULL);
+		mlt_properties_set_data(properties, "f0r_destruct", f0r_destruct , sizeof( f0r_destruct ),NULL,NULL);
 		mlt_properties_set_data(properties, "f0r_get_plugin_info", f0r_get_plugin_info , sizeof(void*),NULL,NULL);
 		mlt_properties_set_data(properties, "f0r_get_param_info", f0r_get_param_info , sizeof(void*),NULL,NULL);
 		mlt_properties_set_data(properties, "f0r_set_param_value", f0r_set_param_value , sizeof(void*),NULL,NULL);
@@ -379,9 +387,10 @@ MLT_REPOSITORY
 			char* firstname = strtok_r( shortname, ".", &save_firstptr );
 #endif
 			char pluginname[1024]="frei0r.";
-			strcat(pluginname,firstname);
+			if ( firstname )
+				strncat( pluginname, firstname, sizeof( pluginname ) - strlen( pluginname ) -1 );
 
-			if ( mlt_properties_get( blacklist, firstname ) )
+			if ( firstname && mlt_properties_get( blacklist, firstname ) )
 				continue;
 
 			void* handle=dlopen(strcat(name, LIBSUF),RTLD_LAZY);
