@@ -48,10 +48,10 @@ mlt_producer producer_qimage_init( mlt_profile profile, mlt_service_type type, c
 		// Get the properties interface
 		mlt_properties properties = MLT_PRODUCER_PROPERTIES( &self->parent );
 	
-		// Callback registration
-#ifdef USE_KDE
+		// Initialize KDE image plugins
 		init_qimage();
-#endif
+
+		// Callback registration
 		producer->get_frame = producer_get_frame;
 		producer->close = ( mlt_destructor )producer_close;
 
@@ -73,7 +73,6 @@ mlt_producer producer_qimage_init( mlt_profile profile, mlt_service_type type, c
 				mlt_properties frame_properties = MLT_FRAME_PROPERTIES( frame );
 				mlt_properties_set_data( frame_properties, "producer_qimage", self, 0, NULL, NULL );
 				mlt_frame_set_position( frame, mlt_producer_position( producer ) );
-				mlt_properties_set_position( frame_properties, "qimage_position", mlt_producer_position( producer ) );
 				refresh_qimage( self, frame );
 				mlt_cache_item_close( self->qimage_cache );
 				mlt_frame_close( frame );
@@ -103,7 +102,7 @@ static int load_svg( producer_qimage self, mlt_properties properties, const char
 	return result;
 }
 
-static int load_sequence( producer_qimage self, mlt_properties properties, const char *filename )
+static int load_sequence_sprintf( producer_qimage self, mlt_properties properties, const char *filename )
 {
 	int result = 0;
 
@@ -141,7 +140,7 @@ static int load_sequence( producer_qimage self, mlt_properties properties, const
 	return result;
 }
 
-static int load_sequence2( producer_qimage self, mlt_properties properties, const char *filename )
+static int load_sequence_deprecated( producer_qimage self, mlt_properties properties, const char *filename )
 {
 	int result = 0;
 	const char *start;
@@ -158,16 +157,39 @@ static int load_sequence2( producer_qimage self, mlt_properties properties, cons
 			strncpy( s, start, n );
 			mlt_properties_set( properties, "begin", s );
 			free( s );
-			s = calloc( 1, strlen( filename ) );
+			s = calloc( 1, strlen( filename ) + 2 );
 			strncpy( s, filename, start - filename );
 			sprintf( s + ( start - filename ), ".%d%s", n, end );
-			result = load_sequence( self, properties, s );
+			result = load_sequence_sprintf( self, properties, s );
 			free( s );
 		}
 	}
 	return result;
 }
 
+static int load_sequence_querystring( producer_qimage self, mlt_properties properties, const char *filename )
+{
+	int result = 0;
+
+	// Obtain filenames with pattern and begin value in query string
+	if ( strchr( filename, '%' ) && strchr( filename, '?' ) )
+	{
+		// Split filename into pattern and query string
+		char *s = strdup( filename );
+		char *querystring = strrchr( s, '?' );
+		*querystring++ = '\0';
+		if ( strstr( filename, "begin=" ) )
+			mlt_properties_set( properties, "begin", strstr( querystring, "begin=" ) + 6 );
+		else if ( strstr( filename, "begin:" ) )
+			mlt_properties_set( properties, "begin", strstr( querystring, "begin:" ) + 6 );
+		// Coerce to an int value so serialization does not have any extra query string cruft
+		mlt_properties_set_int( properties, "begin", mlt_properties_get_int( properties, "begin" ) );
+		result = load_sequence_sprintf( self, properties, s );
+		free( s );
+	}
+	return result;
+}
+
 static int load_folder( producer_qimage self, mlt_properties properties, const char *filename )
 {
 	int result = 0;
@@ -196,8 +218,9 @@ static void load_filenames( producer_qimage self, mlt_properties properties )
 	self->filenames = mlt_properties_new( );
 
 	if (!load_svg( self, properties, filename ) &&
-		!load_sequence( self, properties, filename ) &&
-		!load_sequence2( self, properties, filename ) &&
+		!load_sequence_querystring( self, properties, filename ) &&
+		!load_sequence_sprintf( self, properties, filename ) &&
+		!load_sequence_deprecated( self, properties, filename ) &&
 		!load_folder( self, properties, filename ) )
 	{
 		mlt_properties_set( self->filenames, "0", filename );
@@ -294,9 +317,6 @@ static int producer_get_frame( mlt_producer producer, mlt_frame_ptr frame, int i
 		// Update timecode on the frame we're creating
 		mlt_frame_set_position( *frame, mlt_producer_position( producer ) );
 
-		// Ensure that we have a way to obtain the position in the get_image
-		mlt_properties_set_position( properties, "qimage_position", mlt_producer_position( producer ) );
-
 		// Refresh the image
 		self->qimage_cache = mlt_service_cache_get( MLT_PRODUCER_SERVICE( producer ), "qimage.qimage" );
 		self->qimage = mlt_cache_item_data( self->qimage_cache, NULL );
