@@ -160,8 +160,11 @@ static mlt_service context_pop_service( deserialise_context context, enum servic
 		if ( type != NULL )
 			*type = context->stack_types[ context->stack_service_size ];
 		// Set the service's profile and locale so mlt_property time-to-position conversions can get fps
-		mlt_properties_set_data( MLT_SERVICE_PROPERTIES( result ), "_profile", context->profile, 0, NULL, NULL );
-		mlt_properties_set_lcnumeric( MLT_SERVICE_PROPERTIES( result ), context->lc_numeric );
+		if ( result )
+		{
+			mlt_properties_set_data( MLT_SERVICE_PROPERTIES( result ), "_profile", context->profile, 0, NULL, NULL );
+			mlt_properties_set_lcnumeric( MLT_SERVICE_PROPERTIES( result ), context->lc_numeric );
+		}
 	}
 	return result;
 }
@@ -579,6 +582,7 @@ static void on_end_producer( deserialise_context context, const xmlChar *name )
 		if ( !producer )
 		{
 			mlt_service_close( service );
+			free( service );
 			return;
 		}
 
@@ -665,8 +669,12 @@ static void on_end_producer( deserialise_context context, const xmlChar *name )
 			// Push the producer onto the stack
 			context_push_service( context, producer, mlt_producer_type );
 		}
+	}
 
+	if ( service )
+	{
 		mlt_service_close( service );
+		free( service );
 	}
 }
 
@@ -860,13 +868,17 @@ static void on_end_track( deserialise_context context, const xmlChar *name )
 
 		if ( parent != NULL )
 			context_push_service( context, parent, parent_type );
-
-		mlt_service_close( track );
 	}
 	else
 	{
 		mlt_log_error( NULL, "[producer_xml] Invalid state at end of track\n" );
 	}
+
+	if ( track )
+	{
+		mlt_service_close( track );
+		free( track );
+	}
 }
 
 static void on_start_filter( deserialise_context context, const xmlChar *name, const xmlChar **atts)
@@ -905,6 +917,7 @@ static void on_end_filter( deserialise_context context, const xmlChar *name )
 			if ( parent )
 				context_push_service( context, parent, parent_type );
 			mlt_service_close( service );
+			free( service );
 			return;
 		}
 
@@ -945,14 +958,17 @@ static void on_end_filter( deserialise_context context, const xmlChar *name )
 		{
 			mlt_log_error( NULL, "[producer_xml] filter closed with invalid parent...\n" );
 		}
-
-		// Close the dummy filter service
-		mlt_service_close( service );
 	}
 	else
 	{
 		mlt_log_error( NULL, "[producer_xml] Invalid top of stack on filter close\n" );
 	}
+
+	if ( service )
+	{
+		mlt_service_close( service );
+		free(service);
+	}
 }
 
 static void on_start_transition( deserialise_context context, const xmlChar *name, const xmlChar **atts)
@@ -991,6 +1007,7 @@ static void on_end_transition( deserialise_context context, const xmlChar *name
 			if ( parent )
 				context_push_service( context, parent, parent_type );
 			mlt_service_close( service );
+			free( service );
 			return;
 		}
 		track_service( context->destructors, effect, (mlt_destructor) mlt_transition_close );
@@ -1035,24 +1052,29 @@ static void on_end_transition( deserialise_context context, const xmlChar *name
 			mlt_log_error( NULL, "[producer_xml] transition closed with invalid parent...\n" );
 		}
 
-		// Close the dummy filter service
-		mlt_service_close( service );
 	}
 	else
 	{
 		mlt_log_error( NULL, "[producer_xml] Invalid top of stack on transition close\n" );
 	}
+
+	if ( service )
+	{
+		mlt_service_close( service );
+		free( service );
+	}
 }
 
 static void on_start_consumer( deserialise_context context, const xmlChar *name, const xmlChar **atts)
 {
 	if ( context->pass == 1 )
 	{
-		mlt_consumer consumer = mlt_consumer_new( context->profile );
-		mlt_properties properties = MLT_CONSUMER_PROPERTIES( consumer );
+		mlt_service service = calloc( 1, sizeof( struct mlt_service_s ) );
+		mlt_service_init( service, NULL );
+		mlt_properties properties = MLT_SERVICE_PROPERTIES( service );
 
 		mlt_properties_set_lcnumeric( properties, context->lc_numeric );
-		context_push_service( context, MLT_CONSUMER_SERVICE(consumer), mlt_dummy_consumer_type );
+		context_push_service( context, service, mlt_dummy_consumer_type );
 
 		// Set the properties from attributes
 		for ( ; atts != NULL && *atts != NULL; atts += 2 )
@@ -1111,10 +1133,14 @@ static void on_end_consumer( deserialise_context context, const xmlChar *name )
 					// Inherit the properties
 					mlt_properties_inherit( MLT_CONSUMER_PROPERTIES(context->consumer), properties );
 				}
-				// Close the dummy
-				mlt_service_close( service );
 			}
 		}
+		// Close the dummy
+		if ( service )
+		{
+			mlt_service_close( service );
+			free( service );
+		}
 	}
 }
 
@@ -1532,20 +1558,20 @@ static int file_exists( char *file )
 
 mlt_producer producer_xml_init( mlt_profile profile, mlt_service_type servtype, const char *id, char *data )
 {
-	xmlSAXHandler *sax;
+	xmlSAXHandler *sax, *sax_orig;
 	struct deserialise_context_s *context;
 	mlt_properties properties = NULL;
 	int i = 0;
 	struct _xmlParserCtxt *xmlcontext;
 	int well_formed = 0;
 	char *filename = NULL;
-	int info = strcmp( id, "xml-string" ) ? 0 : 1;
+	int is_filename = strcmp( id, "xml-string" );
 
 	// Strip file:// prefix
 	if ( data && strlen( data ) >= 7 && strncmp( data, "file://", 7 ) == 0 )
 		data += 7;
 
-	if ( data == NULL || !strcmp( data, "" ) || ( info == 0 && !file_exists( data ) ) )
+	if ( data == NULL || !strcmp( data, "" ) || ( is_filename && !file_exists( data ) ) )
 		return NULL;
 
 	context = calloc( 1, sizeof( struct deserialise_context_s ) );
@@ -1560,7 +1586,7 @@ mlt_producer producer_xml_init( mlt_profile profile, mlt_service_type servtype,
 
 	// Decode URL and parse parameters
 	mlt_properties_set( context->producer_map, "root", "" );
-	if ( info == 0 )
+	if ( is_filename )
 	{
 		filename = strdup( data );
 		parse_url( context->params, url_decode( filename, data ) );
@@ -1601,7 +1627,7 @@ mlt_producer producer_xml_init( mlt_profile profile, mlt_service_type servtype,
 	xmlSubstituteEntitiesDefault( 1 );
 	// This is used to facilitate entity substitution in the SAX parser
 	context->entity_doc = xmlNewDoc( _x("1.0") );
-	if ( info == 0 )
+	if ( is_filename )
 		xmlcontext = xmlCreateFileParserCtxt( filename );
 	else
 		xmlcontext = xmlCreateMemoryParserCtxt( data, strlen( data ) );
@@ -1619,14 +1645,17 @@ mlt_producer producer_xml_init( mlt_profile profile, mlt_service_type servtype,
 	}
 
 	// Parse
+	sax_orig = xmlcontext->sax;
 	xmlcontext->sax = sax;
 	xmlcontext->_private = ( void* )context;	
 	xmlParseDocument( xmlcontext );
 	well_formed = xmlcontext->wellFormed;
 	
 	// Cleanup after parsing
-	xmlcontext->sax = NULL;
+	xmlcontext->sax = sax_orig;
 	xmlcontext->_private = NULL;
+	if ( xmlcontext->myDoc )
+		xmlFreeDoc( xmlcontext->myDoc );
 	xmlFreeParserCtxt( xmlcontext );
 	context->stack_node_size = 0;
 	context->stack_service_size = 0;
@@ -1646,7 +1675,7 @@ mlt_producer producer_xml_init( mlt_profile profile, mlt_service_type servtype,
 
 	// Setup the second pass
 	context->pass ++;
-	if ( info == 0 )
+	if ( is_filename )
 		xmlcontext = xmlCreateFileParserCtxt( filename );
 	else
 		xmlcontext = xmlCreateMemoryParserCtxt( data, strlen( data ) );
@@ -1673,6 +1702,7 @@ mlt_producer producer_xml_init( mlt_profile profile, mlt_service_type servtype,
 	sax->getEntity = on_get_entity;
 
 	// Parse
+	sax_orig = xmlcontext->sax;
 	xmlcontext->sax = sax;
 	xmlcontext->_private = ( void* )context;
 	xmlParseDocument( xmlcontext );
@@ -1682,6 +1712,11 @@ mlt_producer producer_xml_init( mlt_profile profile, mlt_service_type servtype,
 	xmlFreeDoc( context->entity_doc );
 	free( sax );
 	xmlMemoryDump( ); // for debugging
+	xmlcontext->sax = sax_orig;
+	xmlcontext->_private = NULL;
+	if ( xmlcontext->myDoc )
+		xmlFreeDoc( xmlcontext->myDoc );
+	xmlFreeParserCtxt( xmlcontext );
 
 	// Get the last producer on the stack
 	enum service_type type;
@@ -1734,8 +1769,17 @@ mlt_producer producer_xml_init( mlt_profile profile, mlt_service_type servtype,
 		if ( getenv( "MLT_XML_DEEP" ) == NULL )
 		{
 			// Now assign additional properties
-			if ( info == 0 && !mlt_properties_get( properties, "resource" ) )
+			if ( is_filename && (
+				mlt_service_identify( service ) == tractor_type ||
+				mlt_service_identify( service ) == playlist_type ||
+				mlt_service_identify( service ) == multitrack_type ) )
+			{
+				mlt_properties_set_int( properties, "_original_type",
+					mlt_service_identify( service ) );
+				mlt_properties_set( properties, "_original_resource",
+					mlt_properties_get( properties, "resource" ) );
 				mlt_properties_set( properties, "resource", data );
+			}
 
 			// This tells consumer_xml not to deep copy
 			mlt_properties_set( properties, "xml", "was here" );
